///////////////////////////////////////////////////////////////////////
// class cIFS_Cube: 
///////////////////////////////////////////////////////////////////////

#include "Cube.h"
#include "affine.h"
#include "Draw_clip.h"
#include "math.h"

cIFS_Cube::cIFS_Cube()
{ this->newsize(50);
}

cIFS_Cube::cIFS_Cube(int size)
{ this->newsize(size);
}

void cIFS_Cube::newsize(int size)
{ int i, j, k;

  for (i = -1; i < 2; i+=2)
    for (j = -1; j < 2; j+=2)
	  for (k = -1; k < 2; k+=2)
	  { m_corners[(i+1)*2 + (j+1) + (k+1)/2][0] = i * size;
	    m_corners[(i+1)*2 + (j+1) + (k+1)/2][1] = j * size;
		m_corners[(i+1)*2 + (j+1) + (k+1)/2][2] = k * size;
	  }
  m_size = size;
}

void cIFS_Cube::transform(double *mat, double *vec)
{
  for (int i = 0; i < 8; i++)
  { mul_mat_vec_self(mat, m_corners[i]);
    add_vectors_self(vec, m_corners[i]);
  }
}

void cIFS_Cube::draw(Screen *s, int32 zoom, ulong color)
{ int i, j, mid_x, mid_y, px1, py1, px2, py2;
  ulong col;

  mid_x = (s->clipLeft + s->clipRight)/2;
  mid_y = (s->clipUp + s->clipDown)/2;

  for( i = 0; i < 7; i++)
    for (j = i+1; j < 8; j++)
	{
	  col = 0;	 	 
	  if (i == 0)
	  { if (j == 1) col = 0x7F7F7F | (0xFF<<16);
	    if (j == 2) col = 0x7F7F7F | (0xFF<<8);
		if (j == 4) col = 0x7F7F7F |  0xFF;
	  }
	  else if (((i ^ j) == 1) ||
	           ((i ^ j) == 2) ||
			   ((i ^ j) == 4))
	    col = 0x7F7F7F;
	  
	  if (col)
	  {
		px1 = (int)(mid_x + m_corners[i][0] * 256 / (m_corners[i][2] + zoom));
		py1 = (int)(mid_y + m_corners[i][1] * 256 / (m_corners[i][2] + zoom));
		px2 = (int)(mid_x + m_corners[j][0] * 256 / (m_corners[j][2] + zoom));
		py2 = (int)(mid_y + m_corners[j][1] * 256 / (m_corners[j][2] + zoom));

		if (i != 0) col = color;
		Bresenham_Line_Clip(*s, px1, py1, px2, py2, col);
	  }
	}
}

double cIFS_Cube::edge_size(int nr)
{ int pt;
  double t1, t2, t3;

  switch (nr) {
    case 0: pt = 4; break;
	case 1: pt = 2; break;
	case 2: pt = 1; break;
  }

  t1 = m_corners[0][0] - m_corners[pt][0];
  t2 = m_corners[0][1] - m_corners[pt][1];
  t3 = m_corners[0][2] - m_corners[pt][2];
  
  return sqrt(t1*t1 + t2*t2 + t3*t3);
}

double cIFS_Cube::angle(int nr)
{ double axis[3], edge[3];

  for (int i = 0; i < 3; i++)
  { axis[i] = 0;
    edge[i] = m_corners[1<<(2-nr)][i] - m_corners[0][i];
  }
  axis[nr] = 1;

  return angle_vectors(axis, edge);
}