/**** Various drawing primitives, with clipping *******************************/

#include "Draw_Clip.h"

void Horizontal_Line_Clip(const Screen& s, int x, int y, int length, int32 color)
{
  if ( y >= s.clipUp       &&
       y <= s.clipDown     &&
	   x <= s.clipRight	   &&
	   x + length > s.clipLeft) // length is not zero-based, so use > iso >=
  {
    if (x < s.clipLeft)
	{
	  length -= s.clipLeft - x;
	  x += s.clipLeft - x;
	}
	if (x+length-1 > s.clipRight)
	  length -= x + length-1 - s.clipRight;

	Horizontal_Line(s, x, y, length, color);
  }
}

void Vertical_Line_Clip(const Screen& s, int x, int y, int length, int32 color)
{ 
  if ( x >= s.clipLeft	   &&
       x <= s.clipRight	   &&
	   y <= s.clipDown	   &&
	   y + length > s.clipUp) // length is not zero-based, so use > iso >=
  {
    if (y < s.clipUp)
	{
	  length -= s.clipUp - y;
	  y += s.clipUp - y;
	}
	if (y+length-1 > s.clipDown)
	  length -= y + length-1 - s.clipDown;

	Vertical_Line(s, x, y, length, color);
  }
}

void Draw_Box_Clip(const Screen& s, int x, int y, int lengthX, int lengthY, int32 color)
{
  if ( x <= s.clipRight		    &&
	   x + lengthX > s.clipLeft && // length is not zero-based, so use > iso >=
	   y <= s.clipDown		    &&
	   y + lengthY > s.clipUp )	   // length is not zero-based, so use > iso >=

  { if (x < s.clipLeft)
	{
	  lengthX -= s.clipLeft - x;
	  x += s.clipLeft - x;
	}
	if (x+lengthX -1 > s.clipRight)
	  lengthX -= x + lengthY - 1 - s.clipRight;

	if (y < s.clipUp)
	{
	  lengthY -= s.clipUp - y;
	  y += s.clipUp - y;
	}
	if (y+lengthY -1 > s.clipDown)
	  lengthY -= y + lengthY - 1 - s.clipDown;

	Draw_Box(s, x, y, lengthX, lengthY, color);
  }
}

// Slow but precise, due to 0.5 rounding correction.

void Bresenham_Line_Clip(const Screen& s, int xstart,int ystart,
								     int xend, int yend, int32 color)
{ 
  int temp;

//ystart wil be top-most point
  if (ystart > yend)
  { temp=xstart; xstart=xend; xend=temp;
    temp=ystart; ystart=yend; yend=temp;
  }

//check of ystart
  if (ystart < s.clipUp)		// point is above clip area
  { if (yend==ystart) return;	// line is horizontal: don't draw
    xstart = (int)((s.clipUp-ystart)*(xend-xstart)/(double)(yend-ystart) +0.5 + xstart);
    ystart = s.clipUp;
  }

  if (yend > s.clipDown)		// point is below clip area
  { if (yend==ystart) return;	// line is horizontal: don't draw
    xend = (int)((s.clipDown-ystart)*(xend-xstart)/(double)(yend-ystart) + 0.5 + xstart);
    yend = s.clipDown;
  }

  //xstart will be left-most point
  if (xstart > xend)
  { temp=xstart; xstart=xend; xend=temp;
    temp=ystart; ystart=yend; yend=temp;
  }

  if (xstart < s.clipLeft)		// point is to the left of clip area
  { if (xend==xstart) return;	// line is vertical: don't draw

    ystart = (int)((s.clipLeft-xstart)*(yend-ystart)/(double)(xend-xstart) +0.5 + ystart);
    xstart = s.clipLeft;
  }

  if (xend > s.clipRight)		// point is to the right of clip area
  { if (xend==xstart) return;	// line is vertical: don't draw

    yend = (int)((s.clipRight-xstart)*(yend-ystart)/(double)(xend-xstart) +0.5 + ystart);
    xend = s.clipRight;
  }

  if (xstart >= s.clipLeft && xstart <= s.clipRight &&
      xend   >= s.clipLeft && xend   <= s.clipRight &&
      ystart >= s.clipUp   && ystart <= s.clipDown  &&
	  yend   >= s.clipUp   && yend   <= s.clipDown)
	  Bresenham_Line(s, xstart, ystart, xend, yend, color);
}