/**** Various drawing primitives **************************************************/

#include "Draw_prim.h"

void Horizontal_Line(const Screen& s, int x, int y, int length, int32 color)
{
  int32 *dest = s.start + y*s.pitch + x;

  for (;length--;)
    *(dest++) = color;
}

void Vertical_Line(const Screen& s, int x, int y, int length, int32 color)
{
  int32 *dest = s.start + y*s.pitch + x;
 
  for (; length--; dest += s.pitch)
    *dest = color;
}

void Draw_Box(const Screen& s, int x, int y, int lengthX, int lengthY, int32 color)
{ 
  int32* tempScreen;
  int tempX;

  for(;lengthY--; y++)
  { 
    tempScreen = s.start + y*s.pitch + x;

    for(tempX = lengthX; tempX--;)
	  *(tempScreen++) = color;
  }
}

void Bresenham_Line(const Screen& s, int xstart,int ystart,
								     int xend, int yend, int32 color)
{ int d,		//decision variable
      xdiff,ydiff,      //line displacements in x and y
      dx_diag,          //diagonal x step for next pixel
      dy_diag,          //diagonal y step for next pixel
	  add_diag,			//diagonal step to reach next pixel
      add_nondiag,		//nondiagonal step to reach next pixel
      diag_inc,		//d's increment for diagonal steps
      nondiag_inc,	//d's increment for nondiagonal steps
      swap;		// swap variable

  int32 *dest = s.start + xstart + ystart*s.pitch;
// Determine drawing direction and step to the next pixel.
  xdiff = xend - xstart;
  ydiff = yend - ystart;
// Determine wether ending point lies to right or left of starting point.
  if (xdiff < 0) 	// Drawing towards smaller x-values?
  {			// Yes, because xdiff is negative
    xdiff = - xdiff;    // Make xdiff positive
    dx_diag = -1;	// And set x-movement accordingly
  } else		// Draw is towards larger x-values
    dx_diag = 1;	// Set x-movement toward larger x.
// Determine wether ending point lies above or below of starting point.
  if (ydiff < 0) 	// Drawing towards smaller y-values?
  {			// Yes, because ydiff is negative
    ydiff = - ydiff;    // Make ydiff positive
    dy_diag = -s.pitch;	// And set y-movement accordingly
  } else		// Draw is towards larger y-values
    dy_diag = s.pitch;		// Set y-movement toward larger y.

// Identify octant containing ending point.
  if (xdiff < ydiff)	// Is y-diff larger then x-diff?
  { swap = xdiff;	// Yes, so swap xdiff and ydiff
    xdiff = ydiff;
    ydiff = swap;
    add_nondiag = dy_diag;
  }				
  else add_nondiag = dx_diag;

  add_diag = dx_diag + dy_diag;

  d = ydiff+ydiff-xdiff;	//initial value for d = 2*ydiff - xdiff
  nondiag_inc = ydiff+ydiff;	//Set initial d increment values
  diag_inc = ydiff+ydiff-xdiff-xdiff;
  for(swap=0;swap<=xdiff;swap++)	// Draw the a+1 pixels.
  { 
	*dest = color;
    if( d < 0)		// is midpoint above the line?
    {				// Yes, step nondiagonaly
      dest += add_nondiag;
      d+= nondiag_inc;	// Update the decision variable.
    }
    else			// Midpoint is below the line, so step diagonaly.
    { dest += add_diag;
      d+= diag_inc;	// Update the decision variable.
    }
  }
}
