///////////////////////////////////////////////////////////////////////
// class cIFS_Interface: 
///////////////////////////////////////////////////////////////////////

#include "stdlib.h"

#include "IFS_Interface.h"
#include "Draw_clip.h"
#include "my_windows.h"
#include "affine.h"
#include "cube.h"
#include "stdio.h"
#include "font.h"
#include "font_6_8.h"
#include "IFS_IO.h"

enum
{
  BORDER_COL = 0x7F7F8F
};

const double IFACE_ROT_STEP = 0.02;

static int frame_nr = 0;

cIFS_Interface* cIFS_Interface::mInstance = 0;

////////////////////////////////////////////////////////////////////////

enum { MAX_RAND = 65536};
int MAX_MASK = MAX_RAND - 1;

int rand_LUT[MAX_RAND];

void Draw_Fractal(cIFS_Fractal *f, Screen *s,
                  int nr_points, int skip_points,
				  double* init_transform, int zoom, int merge_colors)
{
  double t_mat[9], temp[3], point[3] = {0, 0, 0};
  ulong color = 0, prev_color = 0, red, green, blue;
  int i, px, py, r, ran, mid_x, mid_y;

  for (i = 0; i < f->m_nr_ifs; i++)
  { 
    cIFS_Transform *t = &(f->m_ifs[i]);
	make_scale_mat(t->mMatrix, t->GetScale(0), t->GetScale(1), t->GetScale(2));

	make_skewx_mat(t_mat, t->GetSkew(0));
	mul_mat_mat_self(t_mat, t->mMatrix);	  // mat = skew * mat;
	make_skewy_mat(t_mat, t->GetSkew(1));
	mul_mat_mat_self(t_mat, t->mMatrix);	  // mat = skew * mat;
	make_skewz_mat(t_mat, t->GetSkew(2));
	mul_mat_mat_self(t_mat, t->mMatrix);	  // mat = skew * mat;
	mul_mat_mat_self(t->GetRotate(), t->mMatrix);  // mat = rot * skew * mat
  }

// Calculate (and plot) some points

// Skip initial points because they are not yet on the strange attractor.
  for (i = 0; i < skip_points; i++)
  { ran = rand_LUT[i & MAX_MASK];
    r = 0;
	while (ran > f->m_limits[r]) r++;

 	mul_mat_vec_self(f->m_ifs[r].mMatrix, point);
 	add_vectors_self(f->m_ifs[r].GetTrans(), point);
  }

  // Now draw & plot points
  mid_x = (s->clipLeft + s->clipRight)/2;
  mid_y = (s->clipUp + s->clipDown)/2;

  for (i = 0; i < nr_points; i++)
  { 
	ran = rand_LUT[i & MAX_MASK];
    r = 0;
	while (ran > f->m_limits[r]) r++;

 	mul_mat_vec_self(f->m_ifs[r].mMatrix, point);
 	add_vectors_self(f->m_ifs[r].GetTrans(), point);

	// show point

	mul_mat_vec(point, init_transform, temp);

	if ( temp[2] + zoom > 0 )
	{
	  px = (int)(mid_x + temp[0] * 256 / (temp[2] + zoom));
	  py = (int)(mid_y + temp[1] * 256 / (temp[2] + zoom));

	  if ((px >= s->clipLeft) && (px <= s->clipRight) &&
		  (py >= s->clipUp) && (py <= s->clipDown))
      {
		color = f->m_ifs[r].GetColor();

		if (merge_colors)
		{ red = ((color >> 1) & 0x7F0000) + ((prev_color >> 1) & 0x7F0000);
		  green = ((color >> 1) & 0x007F00) + ((prev_color >> 1) & 0x007F00);
		  blue = ((color >> 1) & 0x00007F) + ((prev_color >> 1) & 0x00007F);
		  color = red + green + blue;
		}
		*(s->start + px + py*s->cx) = color;
		prev_color = color;
	  }
	} 
  }
}

void Button_Left()
{
	cIFS_Interface::GetInstance()->Delete_Current_Transform();
}

///////////////////////////////////////////////////////////////////////
cIFS_Interface::cIFS_Interface()
{ 
  int i;

  m_nr_points = 25000;
  m_skip_points = 100;
  m_zoom = 500;
 
  double theTempMat[9];
  make_rotx_mat(m_view_mat, 0.6);
  make_roty_mat(theTempMat, 0.4);
  mul_mat_mat_self(theTempMat, m_view_mat);


  m_view_dx = 0;
  m_view_dy = 0;
  m_view_dz = 0;
  m_view_dxa = 0;
  m_view_dya = 0;
  m_view_dza = 0;
  
  m_select = 0;
  m_command = 0;
  m_merge_colors = 1;
  m_animation = 0;

  sprintf(m_name, "FRACTAL1.IFS");
  m_name_pos = 0;

  for (i = 0; i < MAX_RAND; i++)
    rand_LUT[i] = rand();

  m_f.update_limits(); 
  m_f.SetFullBase();
  m_f.SetFullLimit();
  
  //
  mHelpScreen = false;

  //cButton* theButton = new cButton(0, 400, 20, 20, 0xFF0000, "No!", 0x00FF00,
//						&Button_Left, &Button_Left);
 // mGUI.Add_Button(theButton);

  //int theButtonSize = (s->clipDown - height) - (s->clipUp+length);
//  for (i=0; i < 10; i++)
//  	Draw_Filled_Window(ts1, s->clipLeft + i *theButtonSize, s->clipUp+length,
//   				 theButtonSize , theButtonSize, BORDER_COL);

  if (mInstance)
  {
	delete mInstance;
  }
  mInstance = this;

}

cIFS_Interface::~cIFS_Interface()
{ 
	mInstance = 0;
}

//////////////////////////////////////////////////////////////////////////////////
// Draw
//////////////////////////////////////////////////////////////////////////////////
void cIFS_Interface::Draw(Screen *s)
{ 
	if (mHelpScreen)
	{
		DrawHelpScreen(s);
		return;		
	}
  
  int length, height, i, px1, px2, py1, py2, mid_x, mid_y, dim, com; 
  Screen ts1 = *s, ts2 = *s;
  double v1[3], v2[3], empty[3] = {0, 0, 0};
  cIFS_Cube cube;
  ulong color;
  
  length = (s->clipRight + 1 - s->clipLeft)/2;

// Draw windows
  Draw_Window(ts1, s->clipLeft, s->clipUp, length, length, BORDER_COL);
  Draw_Window(ts1, s->clipLeft + length, s->clipUp, length, length, BORDER_COL);
  height = length/2;

  for ( i = 0; i < 4; i++)
  {
    if (i-1 == m_command % 3) color = 0x7F7F7F | (0xFF << (8*(i-1)));
					   else color = BORDER_COL;
    Draw_Window(ts1, s->clipLeft+height*i, s->clipDown-height, height, height, color);
  }

// Animate fractal if necessary
  if (m_animation)
	m_f.animate(frame_nr++);

// Show Info
  char tstring[256];
  int text_x = s->clipLeft + 5,
      text_y = s->clipDown-height + 5;

  sprintf(tstring, "Nr of IFS transforms: %d", m_f.m_nr_ifs);
  Draw_String(*s, text_x, text_y, capital_font_6_8, tstring, 0xFFFFFF);
  sprintf(tstring, "Nr of IFS points:     %d", m_nr_points);
  Draw_String(*s, text_x, text_y+10, capital_font_6_8, tstring, 0xFFFFFF);
  sprintf(tstring, "Camera distance:      %d", m_zoom);
  Draw_String(*s, text_x, text_y+20, capital_font_6_8, tstring, 0xFFFFFF);
  sprintf(tstring, "Frame nr: %d", frame_nr);
  Draw_String(*s, text_x, text_y+30, capital_font_6_8, tstring, 0xFFFFFF);

  if ((m_command /3) == 0) color = 0xFFFFFF;
					  else color = 0x7F7F7F;
  sprintf(tstring, "Scale");
  Draw_String(*s, text_x, text_y+45, capital_font_6_8, tstring, color);
  if ((m_command /3) == 1) color = 0xFFFFFF;
					  else color = 0x7F7F7F;
  sprintf(tstring, "Rotate");
  Draw_String(*s, text_x, text_y+60, capital_font_6_8, tstring, color);
  if ((m_command /3) == 2) color = 0xFFFFFF;
					  else color = 0x7F7F7F;
  sprintf(tstring, "Skew");
  Draw_String(*s, text_x, text_y+75, capital_font_6_8, tstring, color);
  if ((m_command /3) == 3) color = 0xFFFFFF;
					  else color = 0x7F7F7F;
  sprintf(tstring, "Trans");
  Draw_String(*s, text_x, text_y+90, capital_font_6_8, tstring, color);

  for (i = 0; i < 12; i++)
  { dim = i % 3;
    com = i / 3;

	if (m_f.m_ifs[m_select].mAnim.mBase[com][dim] != 
	    m_f.m_ifs[m_select].mAnim.mLimit[com][dim]) color = 0xAFAFAF;
										  else color = 0x6F6F6F;
    if (m_command == i) color |= 0xFF << (dim*8);
    sprintf(tstring, "%c", 'X'+dim);
    Draw_String(*s, text_x+50 + 20 *dim , text_y+45 + 15*com, capital_font_6_8, tstring, color);
  }

  cIFS_Anim * ani = &m_f.m_ifs[m_select].mAnim;
  dim = m_command % 3;
  com = m_command / 3;

  if (m_f.m_ifs[m_select].mAnim.mBase[com][dim] != 
	    m_f.m_ifs[m_select].mAnim.mLimit[com][dim]) color = 0xAFAFAF;
										  else color = 0x7F7F7F;

  if (m_animation) Draw_String(*s, text_x+90, text_y+110, capital_font_6_8, "<!>", 0xFFFFFF);

  
  if(ani->mSine[com][dim]) sprintf(tstring, "Ipol: Sin");
					   else sprintf(tstring, "Ipol: Lin");  
  Draw_String(*s, text_x, text_y+110, capital_font_6_8, tstring, color);

  sprintf(tstring, "Cycle: %.2d", ani->mCycle[com][dim]);
  Draw_String(*s, text_x, text_y+120, capital_font_6_8, tstring, color);
  sprintf(tstring, "%d", frame_nr % ani->mCycle[com][dim]);
  Draw_String(*s, text_x + 75, text_y+120, capital_font_6_8, tstring, color);
  sprintf(tstring, "Base:  %.2lf", ani->mBase[com][dim]);
  Draw_String(*s, text_x, text_y+130, capital_font_6_8, tstring, color);
  sprintf(tstring, "Limit: %.2lf", ani->mLimit[com][dim]);
  Draw_String(*s, text_x, text_y+140, capital_font_6_8, tstring, color);

  Draw_String(*s, text_x + 0, text_y+160, capital_font_6_8, m_name, 0xFFFFFF);
  Draw_String(*s, text_x + m_name_pos*7, text_y+165, capital_font_6_8, ".", 0xFFFFFF);

  ts1.clipLeft = s->clipLeft + 3;
  ts1.clipUp = s->clipUp + 3;
  ts1.clipRight = s->clipLeft + length - 4;
  ts1.clipDown = s->clipUp + length - 4;

  ts2.clipLeft = s->clipLeft + 3 + length;
  ts2.clipUp = s->clipUp + 3;
  ts2.clipRight = s->clipLeft + 2* length -4;
  ts2.clipDown = s->clipUp + length - 4;

// Draw perspective lines

  double temp_mat[9];
  make_rotx_mat(temp_mat, m_view_dx);
  mul_mat_self_mat(temp_mat, this->m_view_mat);
  make_roty_mat(temp_mat, m_view_dy);
  mul_mat_self_mat(temp_mat, this->m_view_mat);
  make_rotz_mat(temp_mat, m_view_dz);
  mul_mat_self_mat(temp_mat, this->m_view_mat);
  make_rotx_mat(temp_mat, m_view_dxa);
  mul_mat_mat_self(temp_mat, this->m_view_mat);
  make_roty_mat(temp_mat, m_view_dya);
  mul_mat_mat_self(temp_mat, this->m_view_mat);
  make_rotz_mat(temp_mat, m_view_dza);
  mul_mat_mat_self(temp_mat, this->m_view_mat);

  Gram_Schmid_orthonormalize(&(this->m_view_mat[0]),
							 &(this->m_view_mat[3]),
							 &(this->m_view_mat[6]));

  mid_x = (ts1.clipLeft + ts1.clipRight)/2;
  mid_y = (ts1.clipUp + ts1.clipDown)/2;

  for( i = 0; i < 3; i++)
  { v1[0] = 0;  v1[1] = 0;	v1[2] = 0;
    v2[0] = 0;  v2[1] = 0;	v2[2] = 0;
	v1[i] = m_zoom;
	v2[i] =-m_zoom;
    mul_mat_vec_self(this->m_view_mat, v1);
    mul_mat_vec_self(this->m_view_mat, v2);

	px1 = (int)(mid_x + v1[0] * 256 / (v1[2] + m_zoom));
	py1 = (int)(mid_y + v1[1] * 256 / (v1[2] + m_zoom));
	px2 = (int)(mid_x + v2[0] * 256 / (v2[2] + m_zoom));
	py2 = (int)(mid_y + v2[1] * 256 / (v2[2] + m_zoom));

	color = 0x7F7F7F | (0xFF << (8*i));
	Bresenham_Line_Clip(ts1, px1, py1, px2, py2, color);
	Bresenham_Line_Clip(ts2, px1+length, py1, px2+length, py2, color);
  }

// Draw fractal & cubes

  Draw_Fractal(&m_f, &ts1, m_nr_points, m_skip_points, m_view_mat, m_zoom, m_merge_colors);

  for (i = 0; i < m_f.m_nr_ifs; i++)
  {
	cube.newsize(100);
	cube.transform(m_f.m_ifs[i].mMatrix, m_f.m_ifs[i].GetTrans());
	cube.transform(this->m_view_mat, empty);

	if (i == m_select) color = 0xFF0000;
				  else color = 0xFFFFFF;
	cube.draw(&ts2, m_zoom, color);
  }
// Draw small views
  double t_mat[9];

  for (i = 0; i <3; i++)
  { cube.newsize(50);
	cube.transform(m_f.m_ifs[m_select].mMatrix, empty);
	
	ts2.clipLeft = s->clipLeft + 3 + height*(1+i);
    ts2.clipUp = s->clipDown - height + 3;
    ts2.clipRight = s->clipLeft + height*(2+i) -4;
    ts2.clipDown = s->clipDown - 4;
	// Draw edge size
	sprintf(tstring, "%.2f", cube.edge_size(i));
	Draw_String(*s, ts2.clipLeft+2, ts2.clipUp+2, capital_font_6_8, tstring, 0xBFBFBF);
	// draw angles
	sprintf(tstring, "%.2f", cube.angle(i)*180/MATH_PI);
	Draw_String(*s, ts2.clipLeft+2, ts2.clipUp+12, capital_font_6_8, tstring, 0xBFBFBF);	
	// Draw skew factor
	sprintf(tstring, "%.2f", m_f.m_ifs[m_select].GetSkew(i)*180/MATH_PI);
	Draw_String(*s, ts2.clipLeft+2, ts2.clipUp+22, capital_font_6_8, tstring, 0xBFBFBF);
	// Draw vector component
	sprintf(tstring, "%.2f", m_f.m_ifs[m_select].GetTrans(i) );
	Draw_String(*s, ts2.clipLeft+2, ts2.clipUp+32, capital_font_6_8, tstring, 0xBFBFBF);	

	switch(i){
	  case 0: make_normal_mat(t_mat); break;
	  case 1: make_rotx_mat(t_mat, MATH_PI/2); break;
	  case 2: make_roty_mat(t_mat, MATH_PI/2); break;
	}
	cube.transform(t_mat, empty);
	cube.draw(&ts2, m_zoom/2, 0xFFFFFF);
  }
  // Draw buttons
//  int theButtonSize = (s->clipDown - height) - (s->clipUp+length);
//  for (i=0; i < 10; i++)
//  	Draw_Filled_Window(ts1, s->clipLeft + i *theButtonSize, s->clipUp+length,
//   				 theButtonSize , theButtonSize, BORDER_COL);
  
	mGUI.Draw(*s);
}

//////////////////////////////////////////////////////////////////////////////////
// Handle_Key
//////////////////////////////////////////////////////////////////////////////////
void 
cIFS_Interface::Handle_Mouse(SDL_MouseButtonEvent& m)
{ 
	mGUI.Check_Mouse(m.x, m.y, m.button == SDL_BUTTON_LEFT? 1:0);
}

//////////////////////////////////////////////////////////////////////////////////
// Handle_Key
//////////////////////////////////////////////////////////////////////////////////
void cIFS_Interface::Handle_Key(SDL_keysym& k)
{ 
	int do_command = 0;

  	if (mHelpScreen)
  	{ 
  		mHelpScreen = false;		
    }
  	else
  	
  	if (k.mod & KMOD_CTRL )
  	{ switch (k.sym) {

	  case SDLK_SPACE:	Stop_Camera_Motion(); break;
	  case SDLK_KP4:	Stop_Camera_Motion(); 
						m_view_dy -= IFACE_ROT_STEP; break;
	  case SDLK_KP6:	Stop_Camera_Motion();
						m_view_dy += IFACE_ROT_STEP; break;
	  case SDLK_KP8:	Stop_Camera_Motion();
						m_view_dx += IFACE_ROT_STEP; break;
	  case SDLK_KP2:	Stop_Camera_Motion();
						m_view_dx -= IFACE_ROT_STEP; break;
	  case SDLK_KP9:	Stop_Camera_Motion();
						m_view_dz -= IFACE_ROT_STEP; break;
	  case SDLK_KP3:	Stop_Camera_Motion();
						m_view_dz += IFACE_ROT_STEP; break;
	  case SDLK_LEFT:	Stop_Camera_Motion();
						m_view_dya -= IFACE_ROT_STEP; break;
	  case SDLK_RIGHT:	Stop_Camera_Motion();
						m_view_dya += IFACE_ROT_STEP; break;
	  case SDLK_UP:	    Stop_Camera_Motion();
						m_view_dxa -= IFACE_ROT_STEP; break;
	  case SDLK_DOWN:	Stop_Camera_Motion();
						m_view_dxa += IFACE_ROT_STEP; break;
	  case SDLK_PAGEUP: Stop_Camera_Motion();
						m_view_dza -= IFACE_ROT_STEP; break;
	  case SDLK_PAGEDOWN:Stop_Camera_Motion();
						m_view_dza += IFACE_ROT_STEP; break;
	  case SDLK_KP_PLUS: m_zoom -= 50; break;
	  case SDLK_KP_MINUS: m_zoom += 50; break;
	  case SDLK_INSERT : m_f.animate(++frame_nr); break;
	  case SDLK_DELETE : m_f.animate(--frame_nr); break;
	  case SDLK_HOME:	frame_nr = 0; m_f.animate(frame_nr);break;
	}
  }
  else
  if (k.mod & KMOD_SHIFT )
  { switch(k.sym){
      case SDLK_s: save_fractal(&m_f, m_name); break;

      case SDLK_F4: Copy_Current_Transform(); break;
      case SDLK_F5:	Create_New_Default_Transform(); break;
	  case SDLK_F8: Delete_Current_Transform(); break;
					
	  case SDLK_F6: m_nr_points += 250; break;
	  case SDLK_F7: m_nr_points -= 250; break;
	  case SDLK_F11: Reset_To_2_Cubes(); break;
						
	  case SDLK_l:
					   load_fractal(&m_f, m_name);
					   m_select = 0;
					   m_f.update_limits();
	                   break;
	  case SDLK_c : m_merge_colors = (m_merge_colors + 1) & 1; break;

	  case SDLK_SPACE: m_animation = 0;
						  frame_nr = 0;
						  m_f.animate(0);
						  break;

	  case SDLK_COMMA: m_f.SetFullBase(); break;
	  case SDLK_PERIOD:  m_f.SetFullLimit(); break;
	}
  }
  else
  switch(k.sym) {
  	case SDLK_F1: mHelpScreen = true; break;
  	
    case SDLK_UP:	  m_command -= 3; if (m_command < 0) m_command += 12; break;
	case SDLK_DOWN:	  m_command += 3; m_command %= 12; break;
	case SDLK_LEFT:	  m_command--; if (m_command < 0) m_command += 12; break;
	case SDLK_RIGHT:	  m_command++; m_command %= 12; break;
	case SDLK_KP_PLUS:	  do_command = 1; break;
	case SDLK_KP_MINUS:do_command = -1; break;
	case SDLK_TAB:	  m_select = (m_select + 1) % m_f.m_nr_ifs; break;
	case SDLK_CAPSLOCK:m_select = (m_select + m_f.m_nr_ifs - 1) % m_f.m_nr_ifs; break;
	case SDLK_KP_MULTIPLY:do_command = 64; break;
	case SDLK_KP_DIVIDE:  do_command =-64; break;
	case SDLK_BACKSPACE:if (m_name_pos) m_name_pos--; else m_name_pos = 7; break;	
	case SDLK_SPACE:	  m_animation = !m_animation; 
						  frame_nr = 0;
						  break;
	case SDLK_RETURN:	  m_f.switch_interpol(m_select, m_command); break;
	case SDLK_INSERT:  m_f.inc_cycle(m_select, m_command); break;
    case SDLK_DELETE:  m_f.dec_cycle(m_select, m_command); break;
	case SDLK_HOME:	  m_f.set_base(m_select, m_command); break;
    case SDLK_END:	  m_f.set_limit(m_select, m_command); break;
	case SDLK_PAGEUP:  m_f.inc_limit(m_select, m_command); break;
	case SDLK_PAGEDOWN:m_f.dec_limit(m_select, m_command); break;
	case SDLK_COMMA:   m_f.SetFullBase(); break;
	case SDLK_PERIOD:  m_f.SetFullLimit(); break;
	default:
		if ( ( (k.sym >= SDLK_0) && (k.sym <= SDLK_9) ) ||
			 ( (k.sym >= SDLK_a) && (k.sym <= SDLK_z) ) )
			m_name[m_name_pos++] = (unsigned char)k.sym; m_name_pos %=8; break;
			 
  }

  if (do_command)
  {    
    switch (m_command) {
      case 0:
	  case 1:
	  case 2: if (do_command == 64) do_command = 0;
		  m_f.m_ifs[m_select].Transform(cIFS_Transform::eScale,
						m_command%3, do_command);
	          break;
			  	 
	  case 3:
	  case 4:
	  case 5: m_f.m_ifs[m_select].Transform(cIFS_Transform::eRotate,
						m_command%3, do_command);
	          break;

	  case 6:
	  case 7: 
	  case 8: if (do_command == 64) do_command = 0;
			  m_f.m_ifs[m_select].Transform(cIFS_Transform::eSkew,
						m_command%3, do_command);         
	          break;
	  case 9:  
	  case 10: 
	  case 11: m_f.m_ifs[m_select].Transform(cIFS_Transform::eTrans,
						m_command%3, 5*do_command);
			  break;
	}

	m_f.update_limits();
  }
}

//////////////////////////////////////////////////////////////////////////////////
// DrawHelpScreen
//////////////////////////////////////////////////////////////////////////////////
void 
cIFS_Interface::DrawHelpScreen(Screen *s)
{ 
  Screen ts1 = *s;
 
  cIFS_Cube cube;


// Draw window
  Draw_Window(ts1, s->clipLeft, s->clipUp, s->clipRight, s->clipDown, BORDER_COL);  
 
// Show Info
  char tstring[256];
  int text_x = 5,
      text_y = 5;

  sprintf(tstring, "  --- 3D IFS Editor Help --- Abandon all hope, ye who reads this documentation...");
  Draw_String(*s, text_x, text_y, capital_font_6_8, tstring, 0xFFFFFF);
  sprintf(tstring, "* The Windows: The two big windows show the 3D IFS fractal (the cloud of dots at the left), and the");
  Draw_String(*s, text_x, text_y+20, capital_font_6_8, tstring, 0xFFFFFF);
  sprintf(tstring, "transformations that make up this fractal (the cubes on the right). The red cube is the currently selected");
  Draw_String(*s, text_x, text_y+30, capital_font_6_8, tstring, 0xFFFFFF);
  sprintf(tstring, "transformation. The three windows at the bottom show this transformation as seen from the front, side and top.");
  Draw_String(*s, text_x, text_y+40, capital_font_6_8, tstring, 0xFFFFFF);
  
  sprintf(tstring, "The remaining window at the bottom left shows various info, and is used for editing and saving.");
  Draw_String(*s, text_x, text_y+50, capital_font_6_8, tstring, 0xFFFFFF);
  
  sprintf(tstring, "* The edit window:");
  Draw_String(*s, text_x, text_y+70, capital_font_6_8, tstring, 0xFFFFFF);
  
  sprintf(tstring, "Nr of IFS Transforms: an IFS fractal is defined by a number of affine transformation (minimum 2)");
  Draw_String(*s, text_x, text_y+80, capital_font_6_8, tstring, 0xFFFFFF);
  
  sprintf(tstring, "Nr of IFS points: the more points in the cloud, the sharper the fractal, and the more CPU power you need...");
  Draw_String(*s, text_x, text_y+90, capital_font_6_8, tstring, 0xFFFFFF);
  
  sprintf(tstring, "Camera distance: the camera distance for viewing the fractal");
  Draw_String(*s, text_x, text_y+100, capital_font_6_8, tstring, 0xFFFFFF);
  
  sprintf(tstring, "Frame Nr, ipol, cycle, base and Limit are used for animated fractals, but it doesn't work as I want ATM.");
  Draw_String(*s, text_x, text_y+110, capital_font_6_8, tstring, 0xFFFFFF);
  
  sprintf(tstring, "Scale, Rotate, Skew, Trans: this is your current transformation. Use the arrows and keypad plus/minus to ");
  Draw_String(*s, text_x, text_y+120, capital_font_6_8, tstring, 0xFFFFFF);    
  sprintf(tstring, "change it. All other windows will immediately show the updated fractal/transformation.");
  Draw_String(*s, text_x, text_y+130, capital_font_6_8, tstring, 0xFFFFFF);    
  sprintf(tstring, "* Filename: Pressing any letter or number changes the filename, Shift-S/L saves/loads. No overwrite warnings");
  Draw_String(*s, text_x, text_y+140, capital_font_6_8, tstring, 0xFFFFFF);    
  
  sprintf(tstring, "* The Key Combos:");
  Draw_String(*s, text_x, text_y+160, capital_font_6_8, tstring, 0xFFFFFF);    
  sprintf(tstring, "Camera: CTRL-Space stops all motion. CTRL-2/4/6/8/2/9 rotates relative axis, CTRL-arrows/PGUP/PGDN absolute axis");
  Draw_String(*s, text_x, text_y+170, capital_font_6_8, tstring, 0xFFFFFF);
  sprintf(tstring, "        CTRL-Keypad+/Keypad- zooms in/out");
  Draw_String(*s, text_x, text_y+180, capital_font_6_8, tstring, 0xFFFFFF);   
  
  sprintf(tstring, "Transforms: Tab/Capslock selects the next/previous transform. Shift-F4 makes a copy of the current transform.");
  Draw_String(*s, text_x, text_y+190, capital_font_6_8, tstring, 0xFFFFFF); 
  
  sprintf(tstring, "            Shift-F5 add a new default transform. Shift-F8 deletes the current transform. Shift-F11 resets to");
  Draw_String(*s, text_x, text_y+200, capital_font_6_8, tstring, 0xFFFFFF); 
  
  sprintf(tstring, "            2 transforms");
  Draw_String(*s, text_x, text_y+210, capital_font_6_8, tstring, 0xFFFFFF); 
      
  sprintf(tstring, "Varia: Shift-F6/F7 increases/decreases the nr of points. Shift-C flips between merged colors or not");
  Draw_String(*s, text_x, text_y+230, capital_font_6_8, tstring, 0xFFFFFF); 
  
  sprintf(tstring, "You can quit by closing the window with the mouse, but why whould you want to do that?");
  Draw_String(*s, text_x, text_y+250, capital_font_6_8, tstring, 0xFFFFFF); 
  
  sprintf(tstring, "There are several undocumented keys related to animation. Don't press them! (space, enter f.e.)");
  Draw_String(*s, text_x, text_y+260, capital_font_6_8, tstring, 0xFFFFFF); 
}

//////////////////////////////////////////////////////////////////////////////////
// Create_New_Default_Transform
//////////////////////////////////////////////////////////////////////////////////
void	
cIFS_Interface::Create_New_Default_Transform()
{
	if (m_f.m_nr_ifs < MAX_IFS_TRANSFORMS)
	{ 
		int i;
		double theVector[3], theMatrix[9];

		// reset to 2 default cubes
		make_normal_mat(theMatrix); 
		m_f.m_ifs[m_f.m_nr_ifs].SetRotate(theMatrix);

		for (i = 0; i < 3; i++ )
			theVector[i] = 0;
		m_f.m_ifs[m_f.m_nr_ifs].SetSkew(theVector);
		m_f.m_ifs[m_f.m_nr_ifs].SetTrans(theVector);

		for (i = 0; i < 3; i++ )
			theVector[i] = 0.5;
		m_f.m_ifs[m_f.m_nr_ifs].SetScale(theVector);						     
	
		m_f.m_ifs[m_f.m_nr_ifs].SetColor( IFS_colorLUT[m_f.m_nr_ifs] );
		m_f.m_ifs[m_f.m_nr_ifs].SetFullBase();
		m_f.m_ifs[m_f.m_nr_ifs].SetFullBase();
		m_select = m_f.m_nr_ifs;
		m_f.m_nr_ifs ++;
		m_f.update_limits();
	}					
}

//////////////////////////////////////////////////////////////////////////////////
// Delete_Current_Transform
//////////////////////////////////////////////////////////////////////////////////
void	
cIFS_Interface::Delete_Current_Transform()
{
	if (m_f.m_nr_ifs > 2)
	{ 
		m_f.m_ifs[m_select] = m_f.m_ifs[m_f.m_nr_ifs-1];
		m_f.m_ifs[m_select].SetColor( IFS_colorLUT[m_select] );
		m_f.m_nr_ifs --;
		m_select %= m_f.m_nr_ifs;
		m_f.update_limits();
	}
}


//////////////////////////////////////////////////////////////////////////////////
// Reset_To_2_Cubes
//////////////////////////////////////////////////////////////////////////////////
void	
cIFS_Interface::Reset_To_2_Cubes()
{
	int i;
	double theVector[3], theMatrix[9];

	// reset to 2 default cubes
	make_normal_mat(theMatrix); 
	m_f.m_ifs[0].SetRotate(theMatrix);
	m_f.m_ifs[1].SetRotate(theMatrix);

	for (i = 0; i < 3; i++ )
		theVector[i] = 0;

	m_f.m_ifs[0].SetSkew(theVector);
	m_f.m_ifs[1].SetSkew(theVector);
	m_f.m_ifs[0].SetTrans(theVector);
	m_f.m_ifs[1].SetTrans(theVector);

	for (i = 0; i < 3; i++ )
		theVector[i] = 0.5;

	m_f.m_ifs[0].SetScale(theVector);
	m_f.m_ifs[1].SetScale(theVector);

	m_f.m_ifs[0].SetColor( IFS_colorLUT[0] );
	m_f.m_ifs[1].SetColor( IFS_colorLUT[1] );
	m_select = 0;
	m_f.m_nr_ifs = 2;
	m_f.update_limits();
}


//////////////////////////////////////////////////////////////////////////////////
// Copy_Current_Transform
//////////////////////////////////////////////////////////////////////////////////
void	
cIFS_Interface::Copy_Current_Transform()
{
	if (m_f.m_nr_ifs < MAX_IFS_TRANSFORMS)
	{ 
		m_f.m_ifs[m_f.m_nr_ifs] = m_f.m_ifs[m_select];
		m_f.m_ifs[m_f.m_nr_ifs].SetColor( IFS_colorLUT[m_f.m_nr_ifs] );
		m_select = m_f.m_nr_ifs;
		m_f.m_nr_ifs ++;
		m_f.update_limits();						 
	}	
}

//////////////////////////////////////////////////////////////////////////////////
// Stop_Camera_Motion
//////////////////////////////////////////////////////////////////////////////////
void	
cIFS_Interface::Stop_Camera_Motion()
{
	m_view_dy  = 0;
	m_view_dx  = 0;
	m_view_dz  = 0;
	m_view_dxa = 0;
	m_view_dya = 0;
	m_view_dza = 0;
}