//
// IFS Fractal viewer
//

// include files
#include <stdlib.h>
#include "screen.h"
#include "affine.h"
#include "IFS_fractal.h"
#include "IFS_Transform.h"
#include "IFS_IO.h"
#include "time.h"
#include "font.h"
#include "font_6_8.h"
#include "stdio.h"
#include "Asmfx.h"
#include <sys/timeb.h>

/**** Main etc ********************************************************************/

enum { MAX_RAND = 65536};
int MAX_MASK = MAX_RAND - 1;

short int rand_LUT[MAX_RAND];

void Draw_Fractal(cIFS_Fractal *f, Screen *s,
                  int nr_points, int skip_points,
				  double *init_transform, 
				  double *exit_transform, 
				  int zoom, int merge_colors)
{
  double t_mat[9], point[3] = {0, 0, 0};
  ulong color = 0, prev_color = 0, red, green, blue;
  int i, px, py, r, ran, mid_x, mid_y;

  for (i = 0; i < f->m_nr_ifs; i++)
  { 
    cIFS_Transform *t = &(f->m_ifs[i]);
	make_scale_mat(t->m_matrix, t->m_scale[0], t->m_scale[1], t->m_scale[2]);

	make_skewx_mat(t_mat, t->m_skew[0]);
	mul_mat_mat_self(t_mat, t->m_matrix);	  // mat = skewx * mat
	make_skewy_mat(t_mat, t->m_skew[1]);
	mul_mat_mat_self(t_mat, t->m_matrix);	  // mat = skewy * mat
	make_skewz_mat(t_mat, t->m_skew[2]);
	mul_mat_mat_self(t_mat, t->m_matrix);	  // mat = skewz * mat
	mul_mat_mat_self(t->m_rot, t->m_matrix);  // mat = rot * skewxyz * mat;

	mul_mat_self_mat(exit_transform, t->m_matrix); // pre- and post rotation
	mul_mat_mat_self(init_transform, t->m_matrix);
	mul_mat_vec(t->m_trans, init_transform, t->m_vector);
  }

// Calculate (and plot) some points

// Skip initial points because they are not yet on the strange attractor.
  for (i = 0; i < skip_points; i++)
  { 
	  ran = rand_LUT[i & MAX_MASK];
	  r = 0;
	  while (ran > f->m_limits[r]) r++;

 	  mul_mat_vec_self(f->m_ifs[r].m_matrix, point);
 	  add_vectors_self(f->m_ifs[r].m_vector, point);
  }

  // Now draw & plot points

  mid_x = (s->clipLeft + s->clipRight) >> 1;
  mid_y = (s->clipUp + s->clipDown) >> 1;

  for (i = 0; i < nr_points; i++)
  { 
    ran = rand_LUT[i & MAX_MASK];
    r = 0;
	while (ran > f->m_limits[r]) r++;

 	mul_mat_vec_self(f->m_ifs[r].m_matrix, point);
 	add_vectors_self(f->m_ifs[r].m_vector, point);

	if ( point[2] + zoom > 0 )
	{
	  px = (int)(mid_x + point[0] * 256 / (point[2] + zoom));
	  py = (int)(mid_y + point[1] * 256 / (point[2] + zoom));

	  if ((px >= s->clipLeft) && (px <= s->clipRight) &&
		  (py >= s->clipUp) && (py <= s->clipDown))
      {
		color = f->m_ifs[r].m_color;

		if (merge_colors)
		{ red = ((color >> 1) & 0x7F0000) + ((prev_color >> 1) & 0x7F0000);
		  green = ((color >> 1) & 0x007F00) + ((prev_color >> 1) & 0x007F00);
		  blue = ((color >> 1) & 0x00007F) + ((prev_color >> 1) & 0x00007F);
		  color = red + green + blue;
		}
		prev_color = color;
		
		*(s->start + px + py*s->cx) = color;	
	  }
	} 
  }
}

int APIENTRY WinMain(HINSTANCE hInstance,HINSTANCE hPrevInstance,LPSTR lpCmdLine,int nCmdShow)
{
  cIFS_Fractal ifs;
  char filename[50][13] = {"TWIGSAN2.IFS",
                           "STAR5001.IFS",
						   "PYRAMID2.IFS",
						   "TWIGSAN1.IFS"						   
                          };
  int zoom[50] = {400, 400, 400, 400};
 
  int i, max_fracts = 4,
         frac_nr = 0;
  double camera[9], anticamera[9], temp[9], yrot = 0.06, xrot = 0.00;

  make_normal_mat(camera);
  make_normal_mat(anticamera);

  try
  {
    Console console;
    Format format(32,0x00FF0000,0x0000FF00,0x000000FF);
	Format screen_format(16,0xF800,0x7E0,0x1F);
	console.option("DirectX");
	console.option("default width 640");
	console.option("default height 480");
	//console.option("windowed output");
    console.open("IFS View",640, 480, screen_format);
    Surface surface(640, 480, format);

	Screen page;
	
	page.cx = 640;
	page.cy = 480;
	page.clipLeft = 0;
	page.clipRight = 639;
	page.clipUp = 0;
	page.clipDown = 479;

	load_fractal(&ifs, filename[frac_nr]);
	for (i = 0; i < MAX_RAND; i++)
      rand_LUT[i] = rand();
	ifs.update_limits();

	int nr_frames = 0,
	    total_time = 0,
		start_time, stop_time;
	char temp_string[10] = "";
	struct _timeb timebuffer;

	int animation = 1;
   
	Key toets(PTC_KEY_ENTER);
    while (toets != PTC_KEY_ESCAPE)
    {
	  page.start = (int32*)surface.lock();
	  CLEAR_SCREEN((ulong *)page.start, 640*480);

	  // Animate fractal if necessary
	  if (animation)
		for (i = 0; i < ifs.m_nr_ifs; i++)
		  ifs.m_ifs[i].animate();

	  make_roty_mat(temp, yrot);
	  mul_mat_mat_self(temp, camera);
	  make_rotx_mat(temp, xrot);
	  mul_mat_mat_self(temp, camera);


	  make_rotx_mat(temp, -xrot);
	  mul_mat_self_mat(temp, anticamera);
	  make_roty_mat(temp, -yrot);
	  mul_mat_self_mat(temp, anticamera);

	  _ftime(&timebuffer);
	  start_time = (timebuffer.time % 3600)*1000 + timebuffer.millitm;
	  Draw_Fractal(&ifs, &page, 50000, 10, camera, anticamera, zoom[frac_nr], 1);
	  _ftime(&timebuffer);
	  stop_time = (timebuffer.time % 3600)*1000 + timebuffer.millitm;

	  total_time += stop_time - start_time;
	  if(++nr_frames == 10)
	  { _itoa(total_time, temp_string, 10);		
		nr_frames = 0;
		total_time = 0;
	  }
	  Draw_String(page, 5, 5, capital_font_6_8, temp_string, 0xFFFFFF);

	  surface.unlock();
      surface.copy(console);
      console.update();
	  if (console.key())
	  { toets = console.read();

	    if (toets != PTC_KEY_ESCAPE)
		  switch (toets.code())
		  { case PTC_KEY_SPACE: animation = !animation; break;
		    case PTC_KEY_LEFT:  yrot -= 0.03; xrot = 0; break;
			case PTC_KEY_RIGHT: yrot += 0.03; xrot = 0; break;
			case PTC_KEY_UP:    yrot = 0; xrot -= 0.03; break;
			case PTC_KEY_DOWN:  yrot = 0; xrot += 0.03; break;

			default: frac_nr = (frac_nr + 1) % max_fracts;
					 load_fractal(&ifs, filename[frac_nr]);
					 ifs.update_limits(); break;
		  }
	  }
    }
  }
  catch (Error &error)
  { error.report(); }
  return 0;
}
