#ifndef __SEVEN_AFFINE_MATH__
#define __SEVEN_AFFINE_MATH__

/*
  Dot product (a, b, c)*(x, y, z) = a*x + b*y +c*z
	(Dutch: in-product or inwendig product)

  Cross product (a, b, c)X(x, y, z) = (b*z - c*y, c*x-a*z, a*y-b*x)
	(Dutch: uitwendig product)
*/

// Self: same vector/matrix is used for in/output.
// Normal versions should NEVER use the same vector for in/output

void add_vectors(double *u,  double *v, double *out);
void add_vectors_self(double *u, double *out);
void sub_vectors(double *u, double *v, double *out);
void sub_vectors_self(double *u, double *out);
void scale_vector(double s, double *u, double *out);
void scale_vector_self(double s, double *out);
double dot_product(double *u, double *v);
void cross_product(double *u, double *v, double *out);
void cross_product_self(double *u, double *out);
double vector_length(double *u);
void unit_vector(double *u, double *out);
void unit_vector_self(double *out);
double distance_vectors(double *u, double *v);
double angle_vectors(double *u, double *v);
// projection of u on s
void project_vectors(double *u, double *s, double *out);
// projection of out on s
void project_vectors(double *s, double *out);
// Matrix (Self versions are not particulary usefull, because they use an
//         internal copy)
void mul_mat_vec(double *in, double *matrix, double *out);
void mul_mat_vec_self(double *mat, double *out);
// Out = N * M
void mul_mat_mat(double *m, double *n, double *out);
// out = N * OUT;
void mul_mat_mat_self(double *n, double *out);
// out = OUT * N;
void mul_mat_self_mat(double *n, double *out);

// Matric creation
void make_normal_mat(double* in);
void make_rotz_mat(double* in, double angle);
void make_roty_mat(double* in, double angle);
void make_rotx_mat(double* in, double angle);
void make_scale_mat(double* in, double scale_x, double scale_y, double scale_z);
void make_scalex_mat(double* in, double scale);
void make_scaley_mat(double* in, double scale);
void make_scalez_mat(double* in, double scale);
void make_skewx_mat(double* in, double angle);
void make_skewy_mat(double* in, double angle);
void make_skewz_mat(double* in, double angle);
// Advanced stuff
// u1, u2 and u3 should be lineair independent
void Gram_Schmid_orthonormalize(double *v1, double *v2, double *v3);
void Gram_Schmid_orthogonize(double *v1, double *v2, double *v3);

#endif

